/******************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2002             *
 * by the Xiph.Org Foundation http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

  function: packing variable sized words into an octet stream
  last mod: $Id: bitwise.c 7675 2004-09-01 00:34:39Z xiphmont $

 ********************************************************************/

/* We're 'LSb' endian; if we write a word but read individual bits,
   then we'll read the lsb first */
#include <21369.h>
#include <string.h>
#include <stdlib.h>
#include "ogg.h"
#include "macros.h"

#define BUFFER_INCREMENT 256

extern unsigned char databuffer1[PAGE_SIZE_IN]; /* take 8k out of the data segment, not the stack */
extern unsigned char databuffer2[PAGE_SIZE_IN]; /* take 8k out of the data segment, not the stack */

extern int ExtractPacketInfo(void);
extern void FillBufferFromFlash(int bufno);

extern volatile int flash_offset;
extern volatile int data_ready;
extern volatile int read_ptr;
extern volatile int current_packetsize;
extern volatile int eos;
extern unsigned char *page_ptr;
extern volatile int read_ptr;
extern volatile int packets_in_prev_page;
extern volatile int page_ptr_position;

static const unsigned long mask[]=
{0x00000000,0x00000001,0x00000003,0x00000007,0x0000000f,
 0x0000001f,0x0000003f,0x0000007f,0x000000ff,0x000001ff,
 0x000003ff,0x000007ff,0x00000fff,0x00001fff,0x00003fff,
 0x00007fff,0x0000ffff,0x0001ffff,0x0003ffff,0x0007ffff,
 0x000fffff,0x001fffff,0x003fffff,0x007fffff,0x00ffffff,
 0x01ffffff,0x03ffffff,0x07ffffff,0x0fffffff,0x1fffffff,
 0x3fffffff,0x7fffffff,0xffffffff };
/**
static const unsigned int mask8B[]=
{0x00,0x80,0xc0,0xe0,0xf0,0xf8,0xfc,0xfe,0xff};
**/
void oggpack_writeclear(oggpack_buffer *b)
{
  _ogg_free(b->buffer);
  memset(b,0,sizeof(*b));
}

void oggpack_readinit(oggpack_buffer *b,unsigned char *buf,int buf_position,int bytes)
{
  memset(b,0,sizeof(*b));
  b->buffer=b->ptr=buf;
  b->buff_position=buf_position;
  b->ptr_position=buf_position;
  b->storage=bytes;
}


long oggpack_look(oggpack_buffer *b,int bits)
{
  int temp;	
  int result;
  int temp_size = 0;
  //unsigned char *temp_ptr = 0;
  unsigned char *temp_ptr = 0;
  int temp_ptr_position = 0;
  
  unsigned long ret;
  unsigned long m=mask[bits];

  bits+=b->endbit;

  if(b->endbyte+4>=b->storage)
  {
    /* not the main path */
    if(b->endbyte*8+bits>b->storage*8)
    	return(-1);
  }
  
  if( packets_in_prev_page == 0 ) 
  {
  	temp_ptr = b->ptr;
  	temp_ptr_position = b->ptr_position;
  	
  	ret= (Get_Byte_Data(temp_ptr,temp_ptr_position))>>b->endbit;
  	
  	if(bits>8)
  	{
  		temp_ptr_position++;
		if(temp_ptr_position > 3 )
		{
			temp_ptr_position -= 4;
			temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
		}
  		ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(8-b->endbit);  
  		
    	if(bits>16)
    	{
    	  temp_ptr_position++;
	  	  if(temp_ptr_position > 3 )
		  {
			temp_ptr_position -= 4;
			temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
		  }
	      	
	      ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(16-b->endbit);  
	      if(bits>24)
	      {
	      	temp_ptr_position++;
  			if(temp_ptr_position > 3 )
			{
				temp_ptr_position -= 4;
				temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
			}
			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(24-b->endbit);  
			
			if(bits>32 && b->endbit)
			{
				temp_ptr_position++;
  				if(temp_ptr_position > 3 )
				{
					temp_ptr_position -= 4;
					temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
				}
		  		ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(32-b->endbit);
		  		
			}
	      }
	    }
	  }
  }
  else
  {
  	temp_ptr = b->ptr;
  	temp_ptr_position = b->ptr_position;
  
  	ret=(Get_Byte_Data(temp_ptr,temp_ptr_position))>>b->endbit;
  	
  	if( (b->endbyte+4) < packets_in_prev_page) 
  	{
  		if(bits>8)
  		{
  			temp_ptr_position++;
  			if(temp_ptr_position > 3 )
			{
				temp_ptr_position -= 4;
				temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
			}
  			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(8-b->endbit);  
  			
    		if(bits>16)
	    	{
	    	  temp_ptr_position++;
  			  if(temp_ptr_position > 3 )
			  {
				temp_ptr_position -= 4;
				temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
			  }	
		      ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(16-b->endbit);  
		      
		      if(bits>24)
		      {
		      	temp_ptr_position++;
  				if(temp_ptr_position > 3 )
				{
					temp_ptr_position -= 4;
					temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
				}
				ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(24-b->endbit);  
				
				if(bits>32 && b->endbit)
				{
					temp_ptr_position++;
  					if(temp_ptr_position > 3 )
					{
						temp_ptr_position -= 4;
						temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
					}
			  		ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(32-b->endbit);
				}
		      }
		    }
		  }
  	}
  	else
  	{
  		temp_size = b->endbyte;
//		temp_ptr = b->ptr + 1;
  		if(bits>8)
  		{
  		 	if( temp_size + 1 == packets_in_prev_page )
 			{
 				temp_ptr = page_ptr;
 				temp_ptr_position = page_ptr_position;
  				ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(8-b->endbit);
  				
 			}
  			else
  			{	
  				temp_ptr_position++;
  				if(temp_ptr_position > 3 )
				{
					temp_ptr_position -= 4;
					temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
				}	
  				ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(8-b->endbit);  
  			}
  				
    	if(bits>16)
    	{
    		if( temp_size + 2 == packets_in_prev_page )
	 		{
	 			temp_ptr = page_ptr;
 				temp_ptr_position = page_ptr_position;
	  			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(16-b->endbit);
	 		}
	  		else		
	  		{
	  			temp_ptr_position++;
  				if(temp_ptr_position > 3 )
				{
					temp_ptr_position -= 4;
					temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
				}
	  			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(16-b->endbit);  
	  		}
	  			
		    if(bits>24)
	      	{
	      		if( temp_size + 3 == packets_in_prev_page )
		 		{
		 			temp_ptr = page_ptr;
 					temp_ptr_position = page_ptr_position;
		  			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(24-b->endbit);
			
		 		}
		  		else		
		  		{
		  			temp_ptr_position++;
  					if(temp_ptr_position > 3 )
					{
						temp_ptr_position -= 4;
						temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
					}
	  				ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(24-b->endbit);  
		  		}
	  				
				if(bits>32 && b->endbit)
				{
					if( temp_size + 4 == packets_in_prev_page )
	 				{	
	 					temp_ptr = page_ptr;
 						temp_ptr_position = page_ptr_position;
	  					ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(32-b->endbit);
	 				}
	  				else		
	  				{
	  					temp_ptr_position++;
  						if(temp_ptr_position > 3 )
						{
							temp_ptr_position -= 4;
							temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
						}
	  		  			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(32-b->endbit);
	  				}
				}
	      	}
	     }
	  }
  	}
  	
  }
  return(m&ret);
}
void oggpack_adv(oggpack_buffer *b,int bits)
{
  int temp;
  	
  bits+=b->endbit;
  
//  b->ptr+=bits/8;
  b->ptr_position+= bits/8;
  while(b->ptr_position > 3)
  {
  	b->ptr_position -= 4;
  	b->ptr = circptr(b->ptr,1,databuffer1,INPUT_BUFLEN);
  }
   
  b->endbyte+=bits/8;
  b->endbit=bits&7;
  
  if( packets_in_prev_page > 0 )
  {
  	 if( (b->endbyte) == packets_in_prev_page )
  	 {
  	 	b->ptr = page_ptr ;	
  	 }
  }
}

long oggpack_read(oggpack_buffer *b,int bits)
{
  long ret;
  unsigned long m=mask[bits];
  int temp_size;
  unsigned char *temp_ptr = 0;
  int temp_ptr_position = 0;
  
  bits+=b->endbit;

  if(b->endbyte+4>=b->storage)
  {
    /* not the main path */
    ret=-1L;
    if(b->endbyte*8+bits>b->storage*8)
		goto overflow;   
  }
  
  if( packets_in_prev_page == 0 ) 
  {
  	temp_ptr = b->ptr;
  	temp_ptr_position = b->ptr_position;
  	
  	ret= (Get_Byte_Data(temp_ptr,temp_ptr_position))>>b->endbit;
  	
  	if(bits>8)
  	{
  		temp_ptr_position++;
		if(temp_ptr_position > 3 )
		{
			temp_ptr_position -= 4;
			temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
		}
  		ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(8-b->endbit);  
  		
    	if(bits>16)
    	{
    	  temp_ptr_position++;
	  	  if(temp_ptr_position > 3 )
		  {
			temp_ptr_position -= 4;
			temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
		  }
	      	
	      ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(16-b->endbit);  
	      if(bits>24)
	      {
	      	temp_ptr_position++;
  			if(temp_ptr_position > 3 )
			{
				temp_ptr_position -= 4;
				temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
			}
			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(24-b->endbit);  
			
			if(bits>32 && b->endbit)
			{
				temp_ptr_position++;
  				if(temp_ptr_position > 3 )
				{
					temp_ptr_position -= 4;
					temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
				}
		  		ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(32-b->endbit);
		  		
			}
	      }
	    }
	  }
  }
  else
  {
  	temp_ptr = b->ptr;
  	temp_ptr_position = b->ptr_position;
  
  	ret=(Get_Byte_Data(temp_ptr,temp_ptr_position))>>b->endbit;
  	
  	if( (b->endbyte+4) < packets_in_prev_page) 
  	{
  		if(bits>8)
  		{
  			temp_ptr_position++;
  			if(temp_ptr_position > 3 )
			{
				temp_ptr_position -= 4;
				temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
			}
  			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(8-b->endbit);  
  			
    		if(bits>16)
	    	{
	    	  temp_ptr_position++;
  			  if(temp_ptr_position > 3 )
			  {
				temp_ptr_position -= 4;
				temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
			  }	
		      ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(16-b->endbit);  
		      
		      if(bits>24)
		      {
		      	temp_ptr_position++;
  				if(temp_ptr_position > 3 )
				{
					temp_ptr_position -= 4;
					temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
				}
				ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(24-b->endbit);  
				
				if(bits>32 && b->endbit)
				{
					temp_ptr_position++;
  					if(temp_ptr_position > 3 )
					{
						temp_ptr_position -= 4;
						temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
					}
			  		ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(32-b->endbit);
				}
		      }
		    }
		  }
  	}
  	else
  	{
  		temp_size = b->endbyte;
//		temp_ptr = b->ptr + 1;
  		if(bits>8)
  		{
  		 	if( temp_size + 1 == packets_in_prev_page )
 			{
 				temp_ptr = page_ptr;
 				temp_ptr_position = page_ptr_position;
  				ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(8-b->endbit);
  				
 			}
  			else
  			{	
  				temp_ptr_position++;
  				if(temp_ptr_position > 3 )
				{
					temp_ptr_position -= 4;
					temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
				}	
  				ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(8-b->endbit);  
  			}
  				
    	if(bits>16)
    	{
    		if( temp_size + 2 == packets_in_prev_page )
	 		{
				temp_ptr = page_ptr;
 				temp_ptr_position = page_ptr_position;
	  			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(16-b->endbit);
	 		}
	  		else		
	  		{
	  			temp_ptr_position++;
  				if(temp_ptr_position > 3 )
				{
					temp_ptr_position -= 4;
					temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
				}
	  			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(16-b->endbit);  
	  		}
	  			
		    if(bits>24)
	      	{
	      		if( temp_size + 3 == packets_in_prev_page )
		 		{
		 			temp_ptr = page_ptr;
 					temp_ptr_position = page_ptr_position;
		  			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(24-b->endbit);
			
		 		}
		  		else		
		  		{
		  			temp_ptr_position++;
  					if(temp_ptr_position > 3 )
					{
						temp_ptr_position -= 4;
						temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
					}
	  				ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(24-b->endbit);  
		  		}
	  				
				if(bits>32 && b->endbit)
				{
					if( temp_size + 4 == packets_in_prev_page )
	 				{	
	 					temp_ptr = page_ptr;
 						temp_ptr_position = page_ptr_position;
	  					ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(32-b->endbit);
	 				}
	  				else		
	  				{
	  					temp_ptr_position++;
  						if(temp_ptr_position > 3 )
						{
							temp_ptr_position -= 4;
							temp_ptr = circptr(temp_ptr,1,databuffer1,INPUT_BUFLEN);
						}
	  		  			ret|=(Get_Byte_Data(temp_ptr,temp_ptr_position))<<(32-b->endbit);
	  				}
				}
	      	}
	     }
	  }
  	}
  	
  }
ret&=m;
  
 overflow:

//  b->ptr+=bits/8;
  
  b->ptr_position+= bits/8;
  while(b->ptr_position > 3)
  {
  	b->ptr_position -= 4;
  	b->ptr = circptr(b->ptr,1,databuffer1,INPUT_BUFLEN);
  }

  b->endbyte+=bits/8;
  b->endbit=bits&7;
  
  if( packets_in_prev_page > 0 )
  {
  	 if( (b->endbyte) == packets_in_prev_page )
  	 {
  	 	b->ptr = page_ptr ;	
  	 }
  }
    
  return(ret);
}
#undef BUFFER_INCREMENT
