/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2002             *
 * by the XIPHOPHORUS Company http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

 function: simple example decoder
 last mod: $Id: decoder_example.c 7187 2004-07-20 07:24:27Z xiphmont $

 ********************************************************************/

/* Takes a vorbis bitstream from stdin and writes raw stereo PCM to
   stdout.  Decodes simple and chained OggVorbis files from beginning
   to end.  Vorbisfile.a is somewhat more complex than the code below.  */

/* Note that this is POSIX, not ANSI code */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include <21369.h>
#include "macros.h"
#include "codec.h"

extern void initPLL_SDRAM(void);
extern void InitSRU(void);
extern void Init1835viaSPI(void);
extern void InitSPORT(void);
extern void ExternalPortInit(void);
extern void ExternalPort0Isr(int dummy);
extern void FillBufferFromFlash(int bufno);
extern int ExtractPageInfo(void);
extern int vorbis_synthesis_headers(vorbis_info *vi,vorbis_comment *vc);
extern int vorbis_synthesis_audio(vorbis_block *vb);

extern volatile int flash_offset;
extern volatile int data_ready;
extern volatile int bit_rate;
extern volatile int current_packetsize;
extern volatile int eos;
extern volatile int packets_in_prev_page;
extern volatile int read_ptr;
extern unsigned char *page_ptr;
extern unsigned char *prev_page_ptr;
extern volatile int page_ptr_position;
extern volatile int read_ptr_position;

void SerialPort2Isr(int dummy);
int FloattoInt(float value);

volatile int wait_count = 0;
volatile int xmit_count = 0;
volatile int decode_count = 0;
volatile int no_of_pcm_data = 0;

section("seg_dmda1") ogg_int16_t convbuffer1[PAGE_SIZE_OUT]; /* take 8k out of the data segment, not the stack */
section("seg_dmda1") ogg_int16_t convbuffer2[PAGE_SIZE_OUT]; /* take 8k out of the data segment, not the stack */

section("seg_dmda1") unsigned char databuffer1[PAGE_SIZE_IN]; /* take 8k out of the data segment, not the stack */
section("seg_dmda1") unsigned char databuffer2[PAGE_SIZE_IN]; /* take 8k out of the data segment, not the stack */

volatile int xmit_done = 1;
volatile int flash_offset = 0;
volatile int convsize = 0;
volatile int eos = 0;
volatile int init_sport = 0;
volatile int fill_count = 0;
volatile int data_process_count = 0;

ogg_int16_t *output_ptr = convbuffer1;
volatile int output_filled = 0;
volatile int heapID = 0;

int main(int argc, char **argv )
{

  vorbis_info      vi; /* struct that stores all the static vorbis bitstream
			  settings */
  vorbis_comment   vc; /* stlocalruct that stores all the bitstream user comments */
  vorbis_dsp_state vd; /* central working state for the packet->PCM decoder */
  vorbis_block     vb; /*  working space for packet->PCM decode */

  /********** Decode setup ************/

   	initPLL_SDRAM();
 	InitSRU();

 	Init1835viaSPI();
   	ExternalPortInit();

 	interrupts(SIG_SP2,SerialPort2Isr);
  	interrupts(SIG_EP0,ExternalPort0Isr);

  	while (data_ready == 0);
	data_ready = 0;

	FillBufferFromFlash(2);

	while (data_ready == 0);
	
	eos = 0;
	
	while(1)

	{
		/* submit a 4k block to libvorbis' Ogg layer */
		if( eos != 0 )
			break;

		vorbis_info_init(&vi);
		vorbis_comment_init(&vc);

		if(ExtractPageInfo()<0)
		{
			/* error; stream version mismatch perhaps */
			exit(1);
		}

		if(vorbis_synthesis_headers(&vi,&vc)<0)
		{
			/* error case; not a vorbis header */
			exit(1);
		}

		bit_rate = vi.rate;
		
		convsize = (PAGE_SIZE_OUT)/vi.channels;

		/* OK, got and parsed all three headers. Initialize the Vorbis
		 packet->PCM decoder. */
		vorbis_synthesis_init(&vd,&vi); /* central decode state */
		vorbis_block_init(&vd,&vb);     /* local state for most of the decode
						   so multiple block decodes can
						   proceed in parallel.  We could init
						   multiple vorbis_block structures
						   for vd here */
							   
		/* The rest is just a straight decode loop until end of stream */
		while(1)
		{
			/* we have a packet.  Decode it */
			float **pcm;
			volatile int  samples;
			int temp;
						
			if(vorbis_synthesis_audio(&vb)==0) /* test for success! */
			{
				vorbis_synthesis_blockin(&vd,&vb);
				
				samples = vorbis_synthesis_pcmout(&vd,&pcm);
				
				if( packets_in_prev_page == 0 )
		      	{
		      		page_ptr_position = page_ptr_position + current_packetsize;
		      		while (page_ptr_position > 3 )
		      		{
		      			page_ptr_position -= 4;
						page_ptr = circptr(page_ptr,1,databuffer1,INPUT_BUFLEN);
		      		}
					
		      		read_ptr_position = read_ptr_position + current_packetsize;
		      		while (read_ptr_position > 3 )
		      		{
		      			read_ptr_position -= 4;
						read_ptr++;
		      		}
		      	}
		      	else
		      	{

		      		page_ptr_position = page_ptr_position + current_packetsize - packets_in_prev_page;
		      		while (page_ptr_position > 3 )
		      		{
		      			page_ptr_position -= 4;
						page_ptr = circptr(page_ptr,1,databuffer1,INPUT_BUFLEN);
		      		}
					
		      		read_ptr_position = read_ptr_position + current_packetsize - packets_in_prev_page;
		      		while (read_ptr_position > 3 )
		      		{
		      			read_ptr_position -= 4;
						read_ptr++;
		      		}
		      		packets_in_prev_page = 0;
		      	}

		      	if( read_ptr >= PAGE_SIZE_IN )
		      	{
		      		read_ptr = read_ptr - PAGE_SIZE_IN;
					data_process_count++;
					
		      		if( page_ptr < databuffer2 )
		      		{
		      			while (data_ready == 0);
						data_ready = 0;
						
						FillBufferFromFlash(2);
		      		}
		      		else
		      		{
		      			while (data_ready == 0);
		      			data_ready = 0;
		      			
						FillBufferFromFlash(1);
		      		}
		      	}


		      	/* 	**pcm is a multichannel float vector.  In stereo, for
				 example, pcm[0] is left, and pcm[1] is right.  samples is
				 the size of each channel.  Convert the float values
				 (-1.<=range<=1.) to whatever PCM format and write it out */


				while((samples = vorbis_synthesis_pcmout(&vd,&pcm))>0)
				{
					int j;
					int i;
					int bout=(samples<convsize?samples:convsize);
								
					decode_count++;
					/* convert floats to 16 bit signed ints (host order) and
					   interleave */
					if((init_sport == 1) && ( bout > ((PAGE_SIZE_OUT - output_filled)/2)))
						bout = ((PAGE_SIZE_OUT - output_filled)/2); 
					
						
					for(j = 0; j < bout; j++ )
					{
						for( i = 0; i < vi.channels; i++ )
						{
							*output_ptr = FloattoInt(pcm[i][j]);

			 				output_ptr = circptr(output_ptr,1,convbuffer1,OUTPUT_BUFLEN);
						}
					
					} 
					
					output_filled = output_filled + 2*bout;
						
					if( output_filled >= (PAGE_SIZE_OUT) )
					{
					 	output_filled = output_filled - (PAGE_SIZE_OUT);
					 	fill_count++;
						 	
						if( init_sport == 0 )
					 	{
					 	    init_sport = 1;
					 	    InitSPORT();
					 	}
							
					 	if( xmit_count < fill_count )
					 	{ 
					 		while( xmit_done == 0 )
						  		wait_count++;
							   	
							xmit_done = 0;
					 	}
					 } 
					
					no_of_pcm_data += 2*bout;

					vorbis_synthesis_read(&vd,bout); /* tell libvorbis how
									   many samples we
									   actually consumed */
					samples = vorbis_synthesis_pcmout(&vd,&pcm);
				}
			}
			else
				break;
		}
		
		vorbis_block_clear(&vb);
		vorbis_dsp_clear(&vd);
		vorbis_comment_clear(&vc);
		vorbis_info_clear(&vi);  /* must be called last */

		*pCPSP2B = 0;

		*pSPCTL2 = 0;

	}

	return(0);
}

void SerialPort2Isr(int dummy)
{
    xmit_count++;
	xmit_done = 1;
}

inline int FloattoInt(float value)
{
 	int val;
 	
 	val = value * 32767.f;
 	val = __builtin_clip(val, 32767);
 	
 	return val;   
}

