/********************************************************************************************
**
**  File:   initPLL.c
**  Date:   7-29-05
**  Author: SH
**  Use:    Initialize the DSP PLL for the required CCLK and HCLK rates.
**  Note:   CLKIN will be 24.576 MHz from an external oscillator.  The PLL is programmed
**          to generate a core clock (CCLK) of 331.776 MHz - PLL multiplier = 27 and
**          divider = 2.
**
*********************************************************************************************/
#include <def21369.h>
#include <cdef21369.h>

//#define MHZ_400_SPEED
#define MHZ_331_SPEED
//#define MHZ_270_SPEED

void initPLL_SDRAM(void)
{

/********************************************************************************************/

	int i;
	int pmctlsetting = 0;

#ifdef MHZ_400_SPEED    
    pmctlsetting= SDCKR3|PLLM32|INDIV|DIVEN;
#endif    
    
#ifdef MHZ_331_SPEED    
    pmctlsetting= SDCKR2_5|PLLM27|INDIV|DIVEN;
#endif

#ifdef MHZ_270_SPEED    
	pmctlsetting= SDCKR2|PLLM22|INDIV|DIVEN;
#endif

    *pPMCTL= pmctlsetting;
    pmctlsetting|= PLLBP; //Place the PLL in bypass mode
    pmctlsetting^= DIVEN; //clear the DIVEN bit while placing the PLL in bypass mode.
    *pPMCTL= pmctlsetting;

    //Wait for around 4096 cycles for the pll to lock.
    for (i=0; i<5000; i++)
          asm("nop;");

	pmctlsetting= *pPMCTL; // Reading the PMCTL register will return the DIVEN bit value as zero         
    pmctlsetting ^= PLLBP; //Clear Bypass Mode
    pmctlsetting |= (CLKOUTEN);  //and start clkout
    *pPMCTL= pmctlsetting;


    // Programming SDRAM control registers and enabling SDRAM read optimization
    // CCLK_SDCLK_RATIO= 2.5
    // RDIV = ((f SDCLK X t REF )/NRA) - (tRAS + tRP )
    // (166*(10^6)*64*(10^-3)/4096) - (7+3) = 2583
    
	#define SDMODIFY 1
//	*pSDRRC= (0x814)|(SDMODIFY<<17)|SDROPT;
	*pSDRRC= (0x814);

    //===================================================================
    //
    // Configure SDRAM Control Register (SDCTL) for PART MT48LC4M32B2
    //
    //  SDCL3  : SDRAM CAS Latency= 3 cycles
    //  DSDCLK1: Disable SDRAM Clock 1
    //  SDPSS  : Start SDRAM Power up Sequence
    //  SDCAW8 : SDRAM Bank Column Address Width= 8 bits
    //  SDRAW12: SDRAM Row Address Width= 12 bits
    //  SDTRAS7: SDRAM tRAS Specification. Active Command delay = 7 cycles
    //  SDTRP3 : SDRAM tRP Specification. Precharge delay = 3 cycles.
    //  SDTWR2 : SDRAM tWR Specification. tWR = 2 cycles.
    //  SDTRCD3: SDRAM tRCD Specification. tRCD = 3 cycles.
    //
    //--------------------------------------------------------------------

    *pSDCTL= SDCL3|DSDCLK1|SDTRAS6|SDTRP3|SDCAW8|SDPSS|
					   SDTWR2|SDTRCD3|SDRAW12;

    // Note that MS2 & MS3 pin multiplexed with flag2 & flag3.
    // MSEN bit must be enabled to access SDRAM, but LED7 cannot be driven with sdram
    *pSYSCTL |=MSEN; 

    // Mapping Bank 2 to SDRAM
    // Make sure that jumper is set appropriately so that MS2 is connected to
    // chip select of 16-bit SDRAM device
    *pEPCTL |=B2SD;
    *pEPCTL &= ~(B0SD|B1SD|B3SD);
    
    *pEPCTL &= ~(EPBRROT);
	*pEPCTL |=EPBRCORE;
	
    //===================================================================
    //
    // Configure AMI Control Register (AMICTL0) Bank 0 for the ISSI IS61LV5128
    //
    //  WS2 : Wait States = 2 cycles
    //  HC1  : Bus Hold Cycle (at end of write access)= 1 cycle.
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------

    //SRAM Settings
    *pAMICTL0 = WS2|HC1|AMIEN|BW8|PKDIS;

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL) Bank 1 for the AMD AM29LV08
    //
    //  WS23 : Wait States= 23 cycles
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------

    //Flash Settings
    *pAMICTL1 = AMIEN|BW8|WS23|AMIFLSH;  //programs the AMICTL for addressing of 8-bit physical space
}

