///////////////////////////////////////////////////////////////////////////////////////
//NAME:     initSPORT.c (Block-based Talkthrough)
//DATE:     7/29/05
//PURPOSE:  Talkthrough framework for sending and receiving samples to the AD1835.
//
//USAGE:    This file uses SPORT0 to receive data from the ADC and transmits the
//			data to the DAC's via SPORT1A, SPORT1B, SPORT2A and SPORT2B.
//			DMA Chaining is enabled
//
////////////////////////////////////////////////////////////////////////////////////////

/*
   Here is the mapping between the SPORTS and the DACS
   DSP -> DAC4 : SPORT2B : I2S
*/
#include <def21369.h>
#include <cdef21369.h>
#include "macros.h"
#include "os_types.h"

unsigned int PCI = 0x00080000 ;
unsigned int OFFSET = 0x00080000 ;

extern ogg_int16_t convbuffer1[PAGE_SIZE_OUT]; /* take 8k out of the data segment, not the stack */
extern ogg_int16_t convbuffer2[PAGE_SIZE_OUT]; /* take 8k out of the data segment, not the stack */

int TCB_Block0[4] = { 0, PAGE_SIZE_OUT, 1, 0};
int TCB_Block1[4] = { 0, PAGE_SIZE_OUT, 1, 0};


int core_clock = 331776000;
volatile int bit_rate = 0;
int clk_div;

void InitSPORT(void)
{
	TCB_Block0[0] = (int)TCB_Block1 + 3 - OFFSET + PCI ;
    TCB_Block0[3] = (unsigned int) convbuffer1 - OFFSET;
    
    TCB_Block1[0] = (int)TCB_Block0 + 3 - OFFSET + PCI ;
    TCB_Block1[3] = (unsigned int) convbuffer2 - OFFSET; 
    
    
    //Clear the Mutlichannel control registers
    *pSPMCTL0 = 0;
    *pSPMCTL1 = 0;
    *pSPMCTL2 = 0;
    *pSPCTL0 = 0 ;
    *pSPCTL1 = 0 ;
    *pSPCTL2 = 0 ;
    
/**	clk_div = ( core_clock / ((48 * bit_rate) * 8)) - 1;
	
	clk_div = clk_div & 0xFFFF;
    
    clk_div = 0x00170000 | ( clk_div << 1);
    
    *pDIV2 = clk_div; **/

     //============================================================
    //
    // Configure SPORTs 1 & 2 for output to DACs 1-4
    //
    //------------------------------------------------------------
#ifdef TWENTY_FOUR_BIT	
    *pSPCTL2 |= (SPTRAN | OPMODE | SLEN24 | SPEN_B | SCHEN_B | SDEN_B ) ;
#endif

#ifndef TWENTY_FOUR_BIT	
    *pSPCTL2 |= (SPTRAN | OPMODE | SLEN16 | SPEN_B | SCHEN_B | SDEN_B ) ;
#endif
    
    // write to DAC4
   *pCPSP2B = (unsigned int) TCB_Block0 - OFFSET + 3 + PCI;
}



