/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2002             *
 * by the XIPHOPHORUS Company http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

 function: channel mapping 0 implementation
 last mod: $Id: mapping0.c 7187 2004-07-20 07:24:27Z xiphmont $

 ********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "ogg.h"
#include "codec.h"
#include "codec_internal.h"
#include "codebook.h"
#include "window.h"
#include "registry.h"
#include "psy.h"
#include "misc.h"

extern void FillBufferFromFlash(int bufno);

extern volatile int current_data_buffer;
extern volatile int flash_offset;
extern volatile int data_ready;
extern volatile int read_ptr;
extern volatile int eos;
extern volatile void *ext_pointer;
extern volatile int heapID;


/* simplistic, wasteful way of doing this (unique lookup for each
   mode/submapping); there should be a central repository for
   identical lookups.  That will require minor work, so I'm putting it
   off as low priority.

   Why a lookup for each backend in a given mode?  Because the
   blocksize is set by the mode, and low backend lookups may require
   parameters from other areas of the mode/mapping */

static void mapping0_free_info(vorbis_info_mapping *i)
{
  vorbis_info_mapping0 *info=(vorbis_info_mapping0 *)i;
  if(info)
  {
    memset(info,0,sizeof(*info));
    _ogg_free(info);
  }
}

static int ilog(unsigned int v)
{
  int ret=0;
  if(v)--v;
  while(v)
  {
    ret++;
    v>>=1;
  }
  return(ret);
}

/* also responsible for range checking */
static vorbis_info_mapping *mapping0_unpack(vorbis_info *vi,oggpack_buffer *opb)
{
  int i;
  vorbis_info_mapping0 *info; /**=_ogg_calloc(1,sizeof(*info));**/
  codec_setup_info     *ci=vi->codec_setup;
  heapID = 1;
  info=heap_calloc(heapID,1,sizeof(*info));
  memset(info,0,sizeof(*info));

  
  if(oggpack_read(opb,1))
    info->submaps=oggpack_read(opb,4)+1;
  else
    info->submaps=1;

  if(oggpack_read(opb,1))
  {
    info->coupling_steps=oggpack_read(opb,8)+1;

    for(i=0;i<info->coupling_steps;i++)
    {
      int testM=info->coupling_mag[i]=oggpack_read(opb,ilog(vi->channels));
      int testA=info->coupling_ang[i]=oggpack_read(opb,ilog(vi->channels));

      if(testM<0 || 
	 testA<0 || 
	 testM==testA || 
	 testM>=vi->channels ||
	 testA>=vi->channels) 
	 	goto err_out;
    }

  }

  if(oggpack_read(opb,2)>0)
  	goto err_out; /* 2,3:reserved */
    
  if(info->submaps>1)
  {
    for(i=0;i<vi->channels;i++)
    {
      info->chmuxlist[i]=oggpack_read(opb,4);
      if(info->chmuxlist[i]>=info->submaps)
      	goto err_out;
    }
  }
  for(i=0;i<info->submaps;i++)
  {
    oggpack_read(opb,8); /* time submap unused */
    info->floorsubmap[i]=oggpack_read(opb,8);
    if(info->floorsubmap[i]>=ci->floors)
    	goto err_out;
    info->residuesubmap[i]=oggpack_read(opb,8);
    if(info->residuesubmap[i]>=ci->residues)
    	goto err_out;
  }

  return info;

 err_out:
  mapping0_free_info(info);
  return(NULL);
}

static int mapping0_inverse(vorbis_block *vb,vorbis_info_mapping *l)
{
  vorbis_dsp_state     *vd=vb->vd;
  vorbis_info          *vi=vd->vi;
  codec_setup_info     *ci=vi->codec_setup;
  private_state        *b=vd->backend_state;
  vorbis_info_mapping0 *info=(vorbis_info_mapping0 *)l;
  int hs=ci->halfrate_flag; 

  int                   i,j;
  long                  n=vb->pcmend=ci->blocksizes[vb->W];

  float **pcmbundle=alloca(sizeof(*pcmbundle)*vi->channels);
  int    *zerobundle=alloca(sizeof(*zerobundle)*vi->channels);

  int   *nonzero  =alloca(sizeof(*nonzero)*vi->channels);
  void **floormemo=alloca(sizeof(*floormemo)*vi->channels);
  ext_pointer = *floormemo;
  /* recover the spectral envelope; store it in the PCM vector for now */
  for(i=0;i<vi->channels;i++)
  {
    int submap=info->chmuxlist[i];
    floormemo[i]=_floor_P[ci->floor_type[info->floorsubmap[submap]]]->
      inverse1(vb,b->flr[info->floorsubmap[submap]]);
    if(floormemo[i])
      nonzero[i]=1;
    else
      nonzero[i]=0;      
    memset(vb->pcm[i],0,sizeof(*vb->pcm[i])*n/2);
  }

  /* channel coupling can 'dirty' the nonzero listing */
  for(i=0;i<info->coupling_steps;i++)
  {
    if(nonzero[info->coupling_mag[i]] ||
       nonzero[info->coupling_ang[i]]){
      nonzero[info->coupling_mag[i]]=1; 
      nonzero[info->coupling_ang[i]]=1; 
    }
  }

  /* recover the residue into our working vectors */
  for(i=0;i<info->submaps;i++)
  {
    int ch_in_bundle=0;
    for(j=0;j<vi->channels;j++)
    {
      if(info->chmuxlist[j]==i)
      {
		if(nonzero[j])
	  		zerobundle[ch_in_bundle]=1;
		else
	  		zerobundle[ch_in_bundle]=0;
		pcmbundle[ch_in_bundle++]=vb->pcm[j];
      }
    }

    _residue_P[ci->residue_type[info->residuesubmap[i]]]->
      inverse(vb,b->residue[info->residuesubmap[i]],
	      pcmbundle,zerobundle,ch_in_bundle);
  }

  /* channel coupling */
  for(i=info->coupling_steps-1;i>=0;i--)
  {
    float *pcmM=vb->pcm[info->coupling_mag[i]];
    float *pcmA=vb->pcm[info->coupling_ang[i]];

    for(j=0;j<n/2;j++)
    {
      float mag=pcmM[j];
      float ang=pcmA[j];

      if(mag>0)
		if(ang>0)
		{
	  		pcmM[j]=mag;
	  		pcmA[j]=mag-ang;
		}
		else
		{
	  		pcmA[j]=mag;
	  		pcmM[j]=mag+ang;
		}
      else
		if(ang>0)
		{
	  		pcmM[j]=mag;
	  		pcmA[j]=mag+ang;
		}
		else
		{
	  		pcmA[j]=mag;
	  		pcmM[j]=mag-ang;
		}
    }
  }

  /* compute and apply spectral envelope */
  for(i=0;i<vi->channels;i++)
  {
    float *pcm=vb->pcm[i];
    int submap=info->chmuxlist[i];
    _floor_P[ci->floor_type[info->floorsubmap[submap]]]->
      inverse2(vb,b->flr[info->floorsubmap[submap]],
	       floormemo[i],pcm);
  }

  /* transform the PCM data; takes PCM vector, vb; modifies PCM vector */
  /* only MDCT right now.... */
  for(i=0;i<vi->channels;i++)
  {
    float *pcm=vb->pcm[i];
    mdct_backward(b->transform[vb->W][0],pcm,pcm);
  }
  
  /* all done! */
  return(0);
}

/* export hooks */
/**vorbis_func_mapping mapping0_exportbundle=
{
  &mapping0_pack,
  &mapping0_unpack,
  &mapping0_free_info,
  &mapping0_forward,
  &mapping0_inverse
};**/

vorbis_func_mapping mapping0_exportbundle=
{
  NULL,
  &mapping0_unpack,
  &mapping0_free_info,
  NULL,
  &mapping0_inverse
};

