/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2002             *
 * by the XIPHOPHORUS Company http://www.xiph.org/                  *
 *                                                                  *
 ********************************************************************

 function: single-block PCM synthesis
 last mod: $Id: synthesis.c 7187 2004-07-20 07:24:27Z xiphmont $

 ********************************************************************/

#include <stdio.h>
#include "ogg.h"
#include "codec.h"
#include "codec_internal.h"
#include "registry.h"
#include "misc.h"
#include "os.h"
#include "macros.h"

extern int ExtractPacketInfo(void);

extern volatile int current_packetsize;
extern volatile int eos;
extern volatile int packet_eos;
extern volatile int current_packet;
extern volatile int total_packets;
extern volatile ogg_int64_t granulepos;
extern volatile int packets_in_prev_page;
extern unsigned char *prev_page_ptr; 
extern unsigned char *page_ptr;

extern volatile int heapID;

extern volatile int page_ptr_position;
extern volatile int prev_page_ptr_position;

int vorbis_synthesis_audio(vorbis_block *vb)
{
  vorbis_dsp_state     *vd=vb->vd;
  private_state        *b=vd->backend_state;
  vorbis_info          *vi=vd->vi;
  codec_setup_info     *ci=vi->codec_setup;
  oggpack_buffer       *opb=&vb->opb;
  int                   type,mode,i;
  int bytes;
  int result = 0;

  result = ExtractPacketInfo();
  if(result == -1)
  {
  		eos = 1;
  		return result;
  }
  
   bytes = current_packetsize;
 
  /* first things first.  Make sure decode is ready */
  _vorbis_block_ripcord(vb);
  
  if( packets_in_prev_page == 0 )  	
  {
  	oggpack_readinit(opb,page_ptr,page_ptr_position,bytes);
  }
  else
  {
  	oggpack_readinit(opb,prev_page_ptr,prev_page_ptr_position,bytes);
  }

  /* Check the packet type */
  if(oggpack_read(opb,1)!=0)
  {
    /* Oops.  This is not an audio data packet */
    return(OV_ENOTAUDIO);
  }

  /* read our mode and pre/post windowsize */
  mode=oggpack_read(opb,b->modebits);
  if(mode==-1)return(OV_EBADPACKET);
  
  vb->mode=mode;
  vb->W=ci->mode_param[mode]->blockflag;
  if(vb->W)
  {

    /* this doesn;t get mapped through mode selection as it's used
       only for window selection */
    vb->lW=oggpack_read(opb,1);
    vb->nW=oggpack_read(opb,1);
    if(vb->nW==-1)   return(OV_EBADPACKET);
  }
  else
  {
    vb->lW=0;
    vb->nW=0;
  }
  
  /* more setup */
  vb->granulepos=granulepos;
  vb->sequence=current_packet;
//  vb->eofflag=packet_eos;
  vb->eofflag=0;

  /* alloc pcm passback storage */
  vb->pcmend=ci->blocksizes[vb->W];
  heapID = 1;
  vb->pcm=_vorbis_block_alloc(vb,sizeof(*vb->pcm)*vi->channels);
  for(i=0;i<vi->channels;i++)
    vb->pcm[i]=_vorbis_block_alloc(vb,vb->pcmend*sizeof(*vb->pcm[i]));

  heapID = 0;
  /* unpack_header enforces range checking */
  type=ci->map_type[ci->mode_param[mode]->mapping];

  return(_mapping_P[type]->inverse(vb,ci->map_param[ci->mode_param[mode]->
						   mapping]));
}




